import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

/**
 * A superclass for specific actor classes that add smooth movement functionality.
 * 
 * Movement is based on doubles (rather than ints), and thus generates smoother
 * movement for fast moving actors in high resolution worlds.
 * 
 * The SmoothMover also implements world wrap-around: when moving out of the world at any edge,
 * the actor will enter again at the opposite side.
 * 
 * Requires class Vector.
 * 
 * @author Poul Henriksen (modified by softwhizjx : move())
 */
public abstract class SmoothMover extends Actor
{
    private Vector speed = new Vector();
    
    private double x = 0;
    private double y = 0;
    
    // remember our world to avoid calls to getWorld()
    private World world;
    
    public SmoothMover()
    {
    }
    
    @Override
    public void addedToWorld(World world) {
        this.world = world;
    }
    
    /**
     * Create new thing initialised with given speed.
     */
    public SmoothMover(Vector speed)
    {
        this.speed = speed;
    }
    
    /**
     * Move forward one step. (If out of bound, show self again in the opposide side.)
     */
    public void move() 
    {
        x = x + speed.getX();
        y = y + speed.getY();
        
        World world = getWorld();
        if (world == null) {return;}
        else{
            int width = world.getWidth();
            int height = world.getHeight();
            
            if (outOfWorld() && y < 0){
                setLocation(Greenfoot.getRandomNumber(world.getWidth()), world.getHeight());
            } else if (outOfWorld() && x < 0){
                setLocation(world.getWidth(), Greenfoot.getRandomNumber(world.getHeight()));
            } else if (outOfWorld() && y >= world.getHeight()){
                setLocation(Greenfoot.getRandomNumber(world.getWidth()), 1);
            } else if (outOfWorld() && x >= world.getWidth()){
                setLocation(1, Greenfoot.getRandomNumber(world.getHeight()));
            }
            
        
            setLocation(x, y);
            //MOVE
        }
    }
    /**
     * Move forward one step. (If out of bound, just get out!)
     */
    public void moveFixedBound() 
    {
        x = x + speed.getX();
        y = y + speed.getY();
        
        World world = getWorld();
        if (world == null) {return;}
        else{
            int width = world.getWidth();
            int height = world.getHeight();
            
            if (outOfWorld() && y < 0){
                getWorld().removeObject(this);
            } else if (outOfWorld() && x < 0){
                getWorld().removeObject(this);
            } else if (outOfWorld() && y >= world.getHeight()){
                getWorld().removeObject(this);
            } else if (outOfWorld() && x >= world.getWidth()){
                getWorld().removeObject(this);
            }
            
        
            setLocation(x, y);
            //MOVE
        }
    }
    /**
     * Check out of bound.
     */
    private boolean outOfWorld()
    {
        return    (x >= world.getWidth())
               || (x < 0) 
               || (y >= world.getHeight())
               || (y < 0) ;
    }
    
    /**
     * Providing 'setLocation' based on doubles. This will eventually call
     * the built-in 'setLocation' method in Actor.
     */
    public void setLocation(double x, double y) 
    {
        this.x = x;
        this.y = y;
        super.setLocation((int) x, (int) y);
    }
    
    /**
     * Override the default 'setLocation' method to keep our own coordinates
     * up-to-date.
     */
    public void setLocation(int x, int y) 
    {
        setLocation((double) x, (double) y);
    }

    /**
     * Increase the speed with the given vector.
     */
    public void increaseSpeed(Vector s) 
    {
        speed.add(s);
    }
    
    /**
     * Return the current speed.
     */
    public Vector getSpeed() 
    {
        return speed;
    }
}